package com.xunwang.servicebase.handler;

import com.xunwang.servicebase.utils.R;
import com.xunwang.servicebase.exception.ExceptionUtil;
import com.xunwang.servicebase.exception.XunwangException;
import io.jsonwebtoken.ExpiredJwtException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;

@ControllerAdvice
@Slf4j
public class GlobalExceptionHandler {

    // 全局异常
    @ExceptionHandler({Exception.class})
    @ResponseBody
    public R error(Exception e) {
        e.printStackTrace();
        return R.error().message("执行了全局异常处理");
    }

    // 特定异常
    @ExceptionHandler({ArithmeticException.class})
    @ResponseBody
    public R error(ArithmeticException e) {
        e.printStackTrace();
        return R.error().message("执行了算数异常处理");
    }

    // token过期异常
    @ExceptionHandler({ExpiredJwtException.class})
    @ResponseBody
    public R error(ExpiredJwtException e) {
        e.printStackTrace();
        return R.error().message("登录过期，请重新登录");
    }

    // 自定义异常
    @ExceptionHandler({XunwangException.class})
    @ResponseBody
    public R error(XunwangException e) {
        log.error(ExceptionUtil.getMessage(e));
        e.printStackTrace();
        return R.error().code(e.getCode()).message(e.getMsg());
    }
}
