export default {
	// 开启命名空间
	namespaced: true,
	
	// state 数据
	state: () => ({
		// 登录成功之后的 token 字符串
		token: uni.getStorageSync('token') || '',
		// 用户的基本信息
		userinfo: JSON.parse(uni.getStorageSync('userinfo') || '{}'),
		// 老师认证结果
		authentication: uni.getStorageInfoSync('authentication') || '',
		// 经纬度
		location: uni.getStorageSync('location') || '{}',
		// 推荐人  uid 字符串
		recoUID: uni.getStorageSync('recoUID') || '',
	}),
	// 方法
	mutations: {
		// 更新用户的基本信息
		updateUserInfo(state, userinfo) {
			state.userinfo = userinfo
			// 通过 this.commit() 方法，调用 m_user 模块下的 saveUserInfoToStorage 方法，将 userinfo 对象持久化存储到本地
			this.commit('m_user/saveUserInfoToStorage')
		},
		// 将 userinfo 持久化存储到本地
		saveUserInfoToStorage(state) {
			uni.setStorageSync('userinfo', JSON.stringify(state.userinfo))
		},
		// 更新 token 字符串
		updateToken(state, token) {
			state.token = token
			// 通过 this.commit() 方法，调用 m_user 模块下的 saveTokenToStorage 方法，将 token 字符串持久化存储到本地
			this.commit('m_user/saveTokenToStorage')
		},
		// 将 token 字符串持久化存储到本地
		saveTokenToStorage(state) {
			uni.setStorageSync('token', state.token)
		},
		// 更新老师认证结果
		updateAuthentication(state, message) {
			state.authentication = message
			this.commit('m_user/saveAuthentication')
		},
		// 将老师认证结果存储到本地
		saveAuthentication(state) {
			uni.setStorageSync('authentication', state.authentication)
		},
		// 更新经纬度
		updateLocation(state, location) {
			state.location = location
			this.commit('m_user/saveLocation')
		},
		// 将经纬度存储到本地
		saveLocation(state) {
			uni.setStorageSync('location', state.location)
		},
		// 更新 recoUID 字符串
		updateRecoUID(state, recoUID) {
			state.recoUID = recoUID
			this.commit('m_user/saveRecoUIDToStorage')
		},
		// 将 recoUID 字符串持久化存储到本地
		saveRecoUIDToStorage(state) {
			uni.setStorageSync('recoUID', state.recoUID)
		}
	},
}